#include <list>
#include <iostream>
#include <time.h>
#include "adevs.h"
using namespace adevs;
using namespace std;

static int atomic_number = 0;
static int internal_execs = 0;

struct flagged 
{ 
	bool flag;
};

class SimpleAtomic: public Atomic<char>
{
	public:
		SimpleAtomic():
		Atomic<char>()
		{
			number = atomic_number++;
		}
		void delta_int()
		{
			internal_execs++;
		}
		void delta_ext(double e, const Bag<char>& xb) { assert(false); }
		void delta_conf(const Bag<char>& xb) { assert(false); }
		double ta() { return 1.0; }
		void output_func(Bag<char>& y){}
		void gc_output(Bag<char>& g){}
		bool model_transition() 
		{
			flagged* parent = dynamic_cast<flagged*>(getParent());
			assert(parent->flag == false); 
			return true; 
		}
		~SimpleAtomic(){ atomic_number--; }
	private:
		int number;
};
	
class SimpleNetwork: public Network<char>, public flagged
{
	public:
		SimpleNetwork(int depth = 0):
		Network<char>(),
		flagged(),
		depth(depth)
		{
			flag = false;
			int initial_count = rand()%3+1;
			for (int i = 0; i < initial_count; i++)
			{
				add_model();
			}
		}
		void getComponents(Set<Devs<char>*>& c)
		{
			list<Devs<char>*>::iterator iter;
			for (iter = models.begin(); iter != models.end(); iter++)
			{
				c.insert(*iter);
			}
		}
		void route(const char& value, Devs<char>* model, Bag<Event<char> >& r){}
		bool model_transition()
		{
			flag = true;
			if (getParent() != NULL)
			{
				flagged* parent = dynamic_cast<flagged*>(getParent());
				assert(parent->flag == false); 
			}
			int choice = rand()%3;
			if (choice == 0)
			{
				add_model();
				return true;
			} 
			else if (choice == 2 && !models.empty())
			{
				models.pop_back();
				return true;
			}
			return false;
		}
		~SimpleNetwork()
		{
			list<Devs<char>*>::iterator iter;
			for (iter = models.begin(); iter != models.end(); iter++)
			{
				delete *iter;
			}
		}
	private:
		list<Devs<char>*> models;
		int depth;

		void add_model()
		{
			Devs<char>* model = NULL;
			if (rand()%2 == 0 || depth == 5)
			{
				model = new SimpleAtomic();
			}
			else 
			{
				model = new SimpleNetwork(depth+1);
			}
			assert(model != NULL);
			model->setParent(this);
			models.push_front(model);
		}
};

void reset_flags(SimpleNetwork* model)
{
	model->flag = false;
	Set<Devs<char>*> components;
	model->getComponents(components);
	Set<Devs<char>*>::iterator iter;
	for (iter = components.begin(); iter != components.end(); iter++)
	{
		SimpleNetwork* next_model;
		next_model = dynamic_cast<SimpleNetwork*>(*iter);
		if (next_model != NULL) reset_flags(next_model);
	}
}

int main()
{
	unsigned long seed = time(NULL);
	cout << seed << endl;
	srand(seed);
	SimpleNetwork* model = new SimpleNetwork();
	Simulator<char>* sim = new Simulator<char>(model);
	while (sim->nextEventTime() < DBL_MAX)
	{
		reset_flags(model);
		sim->execNextEvent();
	}
	return 0;
}
